/**
 * OnCradleOn
 * (c) Kenta Cho (cs8k-cyu@asahi-net.or.jp)
 * http://www.asahi-net.or.jp/~cs8k-cyu/
 *
 * oncrdon.c(main routine)
 */

#include <PalmOS.h>
#include "oncrdon.h"
#include "oncrdon_rsc.h"

static PrefStatus prefStatus;

/**
 * get application localID and cardNo
 */
static AppInfo getApplicationDBInfo() {
  AppInfo ai;
  LocalID dbID;
  UInt16 cardNo;
  UInt32 creator = CID;
  DmOpenRef dbR;

  dbR = DmOpenDatabaseByTypeCreator('appl', creator, dmModeReadOnly);
  DmOpenDatabaseInfo(dbR, &dbID, NULL, NULL, &cardNo, NULL);
  DmCloseDatabase(dbR);
  ai.dbID = dbID; ai.cardNo = cardNo;
  return ai;
}

/**
 * regist sleep request notification
 */
static Boolean tryToRegister() {
  AppInfo ai;
  ai = getApplicationDBInfo();
  if ( errNone == SysNotifyRegister(ai.cardNo, ai.dbID, 
				    sysNotifySleepRequestEvent, NULL,
				    sysNotifyNormalPriority, NULL) ) {
    return true;
  } else {
    return false;
  }
}

/**
 * unregist sleep request notification
 */
static Boolean tryToUnregister() {
  AppInfo ai;
  ai = getApplicationDBInfo();
  if ( errNone == SysNotifyUnregister(ai.cardNo, ai.dbID, 
				      sysNotifySleepRequestEvent, 
				      sysNotifyNormalPriority) ) {
    return true;
  } else {
    return false;
  }
}

/**
 * set check box if notification is already registerd
 */
static void setCheckBoxStatus() {
  Boolean exist;
  UInt16 prefsSize = sizeof(prefStatus);
  exist = PrefGetAppPreferences
    (CID, 0, &prefStatus, &prefsSize, true);
  if ( exist ) {
    if ( prefStatus.stayon ) {
      FormType *frm = FrmGetActiveForm();
      UInt16 checkIdx = FrmGetObjectIndex(frm, OCO_CHECKBOX);
      FrmSetControlValue(frm, checkIdx, 1);
    }
  } else {
    prefStatus.stayon = false;
  }
}

/**
 * event hanlder for main form
 */
static Boolean MainFormHandleEvent(EventPtr event) {
  Boolean handled = false;
  switch ( event->eType ) {
  case ctlSelectEvent:
    if ( event->data.ctlSelect.controlID == OCO_CHECKBOX ) {
      if ( event->data.ctlSelect.on ) {
	tryToRegister();
	prefStatus.stayon = true;
      } else {
	tryToUnregister();
	prefStatus.stayon = false;
      }
    }
    handled = true;
    break;
  case frmOpenEvent:
    FrmDrawForm(FrmGetActiveForm());
    setCheckBoxStatus();
    handled = true;
    break;
  }
  return handled;
}

/**
 * activate forms
 */
static Boolean ApplicationHandleEvent(EventPtr event) {
  FormType *frm;
  Int16 formID;
  Boolean handled = false;

  if ( event->eType == frmLoadEvent ) {
    formID = event->data.frmLoad.formID;
    frm = FrmInitForm(formID);
    FrmSetActiveForm(frm);
    switch ( formID ) {
    case MAIN_FORM:
      FrmSetEventHandler(frm, MainFormHandleEvent);
      break;
    }
    handled = true;
  }
  return handled;
}

static void EventLoop(void) {
  EventType event;
  UInt16 error;
  do {
    EvtGetEvent(&event, -1);
    if ( !SysHandleEvent(&event) ) {
      if ( !ApplicationHandleEvent(&event) ) {
	FrmDispatchEvent(&event);
      }
    }
  } while ( event.eType != appStopEvent );
}

static int StartApplication(void) {
  FrmGotoForm(MAIN_FORM);
}

static void StopApplication(void) {
  UInt16 prefsSize = sizeof(prefStatus);
  PrefSetAppPreferences
    (CID, 0, VERSION, &prefStatus, prefsSize, true);
}

UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags) {
  if ( cmd == sysAppLaunchCmdNormalLaunch ) {
    StartApplication();
    EventLoop();
    StopApplication();
  } else if ( cmd == sysAppLaunchCmdNotify ) {
    /* sleep request notify */
    Boolean plgdIn;
    SleepEventParamType *sept 
      = ((SysNotifyParamType*)cmdPBP)->notifyDetailsP;
    SysBatteryInfo(false, NULL, NULL, NULL, NULL, &plgdIn, NULL);
    /* if palm is on cradle and requset is notified by auto off */
    if ( plgdIn && sept->reason == sysSleepAutoOff ) {
      /* defer sleep */
      sept->deferSleep++;
    }
  } else if ( cmd == sysAppLaunchCmdSystemReset ) {
    /* respond system reset */
    PrefStatus stat;
    Boolean exist;
    UInt16 prefsSize = sizeof(stat);
    exist = PrefGetAppPreferences
      (CID, 0, &stat, &prefsSize, true);
    if ( exist ) {
      if ( stat.stayon ) {
	tryToRegister();
      }
    }
  }
  return 0;
}
